@@ -37,21 +37,18 @@ sub Commit {
     my $txn = $self->TransactionObj;
     my $level = $ticket->FirstCustomFieldValue('SLA');
 
-    my ($last_reply, $is_requestor) = $self->LastEffectiveAct;
-    $RT::Logger->debug(
-        'Last effective '. ($is_requestor? '':'non-') .'requestors\' reply'
-        .' to ticket #'. $ticket->id .' is txn #'. $last_reply->id
-    );
+    my $last_reply = $self->LastRequestorsEffectiveAct;
+    $RT::Logger->debug('Last effective requestors\' reply to ticket #'. $ticket->id .' is txn #'. $last_reply->id )
+        if $last_reply;
 
-    my $response_due = $self->Due(
-        Ticket => $ticket,
+    my $response_due;
+    $response_due = $self->Due(
         Level => $level,
-        Type => $is_requestor? 'Response': 'KeepInLoop',
+        Type => 'Response',
         Time => $last_reply->CreatedObj->Unix,
-    );
+    ) if $last_reply;
 
     my $resolve_due = $self->Due(
-        Ticket => $ticket,
         Level => $level,
         Type => 'Resolve',
         Time => $ticket->CreatedObj->Unix,
@@ -77,7 +74,7 @@ sub IsRequestorsAct {
     return $self->TicketObj->Requestors->HasMemberRecursively( $actor )? 1 : 0;
 }
 
-sub LastEffectiveAct {
+sub LastRequestorsEffectiveAct {
     my $self = shift;
 
     my $txns = $self->TicketObj->Transactions;
@@ -90,13 +87,10 @@ sub LastEffectiveAct {
 
     my $res;
     while ( my $txn = $txns->Next ) {
-        unless ( $self->IsRequestorsAct( $txn ) ) {
-            last if $res;
-            return ($txn);
-        }
+        return $res unless $self->IsRequestorsAct( $txn );
         $res = $txn;
     }
-    return ($res, 'requestor');
+    return $res;
 }
 
 1;
