@@ -4,7 +4,7 @@ use warnings;
 
 package RT::Extension::SLA;
 
-our $VERSION = '0.05_02';
+our $VERSION = '0.01';
 
 =head1 NAME
 
@@ -12,41 +12,21 @@ RT::Extension::SLA - Service Level Agreements for RT
 
 =head1 DESCRIPTION
 
-RT extension to implement automated due dates using service levels.
-
-=head1 INSTALL
-
-=over 4
-
-=item perl Makefile.PL
-
-=item make
-
-=item make install
-
-=item make initdb (for the first time only)
-
-=item Base configuration
-
-In RT 3.8 and later, you must enable the plugin by adding RT::Extension::SLA
-to your @Plugins line (or create one) like:
-
-    Set(@Plugins,(qw(RT::Extension::SLA)));
-
-=back
+RT's extension that allows you to automate due dates using
+service levels.
 
 =head1 CONFIGURATION
 
-Service level agreements of tickets is controlled by an SLA custom field (CF).
-This field is created during C<make initdb> step (above) and applied globally.
-This CF MUST be of C<select one value> type. Values of the CF define the
-service levels.
+Service level agreements of tickets is controlled by SLA custom
+field. It's created during `make initdb` step and applied globally.
+This CF MUST be of 'select one value' type. Values of the CF
+define service levels.
 
 It's possible to define different set of levels for different
 queues. You can create several CFs with the same name and
 different set of values. But if you move tickets between
-queues a lot then it's going to be a problem and it's preferred
-to use B<ONE> SLA custom field.
+queues a lot then it's gonna be a problem and it's preferred
+to use ONE SLA custom field.
 
 There is no WebUI in the current version. Almost everything is
 controlled in the RT's config using option C<%RT::ServiceAgreements>
@@ -63,55 +43,34 @@ and C<%RT::ServiceBusinessHours>. For example:
         },
     );
 
-In this example I<Incident> is the name of the queue, and I<2h> is the name of
-the SLA which will be applied to this queue by default.
-
 Each service level can be described using several options:
-L<Starts|/"Starts (interval, first business minute)">,
+L<StartImmediately|/"StartImmediately (boolean, false)">,
 L<Resolve|/"Resolve and Response (interval, no defaults)">,
 L<Response|/"Resolve and Response (interval, no defaults)">,
-L<KeepInLoop|/"Keep in loop (interval, no defaults)">,
 L<OutOfHours|/"OutOfHours (struct, no default)">
-and L<ServiceBusinessHours|/"Configuring business hours">.
+and L</BusinessHours>.
 
-=head2 Starts (interval, first business minute)
+=head2 StartImmediately (boolean, false)
 
-By default when a ticket is created Starts date is set to
+By default when ticket is created Starts date is set to
 first business minute after time of creation. In other
-words if a ticket is created during business hours then
-Starts will be equal to Created time, otherwise Starts will
+words if ticket is created during business hours then
+Starts will be equal to Created time, otherwise it'll
 be beginning of the next business day.
 
 However, if you provide 24/7 support then you most
 probably would be interested in Starts to be always equal
-to Created time.
-
-Starts option can be used to adjust behaviour. Format
-of the option is the same as format for deadlines which
-described later in details. RealMinutes, BusinessMinutes
-options and OutOfHours modifiers can be used here like
-for any other deadline. For example:
-
-    'standard' => {
-        # give people 15 minutes
-        Starts   => { BusinessMinutes => 15  },
-    },
-
-You can still use old option StartImmediately to set
-Starts date equal to Created date.
+to Created time. In this case you can set option
+StartImmediately to true value.
 
 Example:
-
     '24/7' => {
         StartImmediately => 1,
         Response => { RealMinutes => 30 },
     },
-
-But it's the same as:
-
-    '24/7' => {
-        Starts => { RealMinutes => 0 },
-        Response => { RealMinutes => 30 },
+    'standard' => {
+        StartImmediately => 0, # can be ommited as it's default
+        Response => { BusinessMinutes => 2*60 },
     },
 
 =head2 Resolve and Response (interval, no defaults)
@@ -130,13 +89,13 @@ Defines deadline when a ticket should be resolved. This option is
 quite simple and straightforward when used without L</Response>.
 
 Example:
-
     # 8 business hours
     'simple' => { Resolve => 60*8 },
     ...
     # one real week
     'hard' => { Resolve => { RealMinutes => 60*24*7 } },
 
+
 =head3 Response
 
 In many companies providing support service(s) resolve time
@@ -183,7 +142,7 @@ Resolve and Response can be combined. In such case due date is set
 according to the earliest of two deadlines and never is dropped to
 'not set'.
 
-If a ticket met its Resolve deadline then due date stops "flipping",
+If a ticket met its Resolve deadline then due date stops "fliping",
 is freezed and the ticket becomes overdue. Before that moment when
 non-requestor replies to a ticket, due date is changed to Resolve
 deadline instead of 'Not Set', as well this happens when a ticket
@@ -225,23 +184,6 @@ level set deadline to 8 real hours starting from the next business
 day, when tickets with the second level should be resolved in the
 next 8 hours after creation.
 
-=head2 Keep in loop (interval, no defaults)
-
-If response deadline is used then Due date is changed to repsonse
-deadline or to "Not Set" when staff replies to a ticket. In some
-cases you want to keep requestors in loop and keed them up to date
-every few hours. KeepInLoop option can be used to achieve this.
-
-    'incident' => {
-        Response   => { RealMinutes => 60*1  }, # one hour
-        KeepInLoop => { RealMinutes => 60*2 }, # two hours
-        Resolve    => { RealMinutes => 60*24 }, # 24 real hours
-    },
-
-In the above example Due is set to one hour after creation, reply
-of a non-requestor moves Due date two hours forward, requestors'
-replies move Due date to one hour and resolve deadine is 24 hours.
-
 =head2 OutOfHours (struct, no default)
 
 Out of hours modifier. Adds more real or business minutes to resolve
@@ -307,8 +249,7 @@ Default Business Hours setting is in $RT::ServiceBusinessHours{'Default'}.
 
 =head2 Defining service levels per queue
 
-In the config you can set per queue defaults, using:
-
+In the config you can set per queue defaults, using
     %RT::ServiceAgreements = (
         Default => 'global default level of service',
         QueueDefault => {
@@ -344,27 +285,10 @@ sub BusinessHours {
 
 sub Agreement {
     my $self = shift;
-    my %args = (
-        Level => undef,
-        Type => 'Response',
-        Time => undef,
-        Ticket => undef,
-        Queue  => undef,
-        @_
-    );
+    my %args = ( Level => undef, Type => 'Response', Time => undef, @_ );
 
     my $meta = $RT::ServiceAgreements{'Levels'}{ $args{'Level'} };
     return undef unless $meta;
-
-    if ( exists $meta->{'StartImmediately'} || !defined $meta->{'Starts'} ) {
-        $meta->{'Starts'} = {
-            delete $meta->{'StartImmediately'}
-                ? ( )
-                : ( BusinessMinutes => 0 )
-            ,
-        };
-    }
-
     return undef unless $meta->{ $args{'Type'} };
 
     my %res;
@@ -377,73 +301,55 @@ sub Agreement {
         return undef;
     }
 
-    $res{'OutOfHours'} = $meta->{'OutOfHours'}{ $args{'Type'} };
-
-    $args{'Queue'} ||= $args{'Ticket'}->QueueObj if $args{'Ticket'};
-    if ( $args{'Queue'} && ref $RT::ServiceAgreements{'QueueDefault'}{ $args{'Queue'}->Name } ) {
-        $res{'Timezone'} = $RT::ServiceAgreements{'QueueDefault'}{ $args{'Queue'}->Name }{'Timezone'};
+    if ( defined $meta->{'StartImmediately'} ) {
+        $res{'StartImmediately'} = $meta->{'StartImmediately'};
     }
-    $res{'Timezone'} ||= $meta->{'Timezone'} || $RT::Timezone;
 
-    $res{'BusinessHours'} = $meta->{'BusinessHours'};
+    if ( $args{'Time'} and my $tmp = $meta->{'OutOfHours'}{ $args{'Type'} } ) {
+        my $bhours = $self->BusinessHours( $meta->{'BusinessHours'} );
+        if ( $bhours->first_after( $args{'Time'} ) != $args{'Time'} ) {
+            foreach ( qw(RealMinutes BusinessMinutes) ) {
+                next unless $tmp->{ $_ };
+                $res{ $_ } ||= 0;
+                $res{ $_ } += $tmp->{ $_ };
+            }
+        }
+    }
 
     return \%res;
 }
 
 sub Due {
     my $self = shift;
-    return $self->CalculateTime( @_ );
+    my %args = ( Level => undef, Type => undef, Time => undef, @_ );
+
+    my $agreement = $self->Agreement( %args );
+    return undef unless $agreement;
+
+    my $meta = $RT::ServiceAgreements{'Levels'}{ $args{'Level'} };
+
+    my $res = $args{'Time'};
+    if ( defined $agreement->{'BusinessMinutes'} ) {
+        my $bhours = $self->BusinessHours( $meta->{'BusinessHours'} );
+        $res = $bhours->add_seconds( $res, 60 * $agreement->{'BusinessMinutes'} );
+    }
+    $res += 60 * $agreement->{'RealMinutes'}
+        if defined $agreement->{'RealMinutes'};
+
+    return $res;
 }
 
 sub Starts {
     my $self = shift;
-    return $self->CalculateTime( @_, Type => 'Starts' );
-}
+    my %args = ( Level => undef, Time => undef, @_ );
 
-sub CalculateTime {
-    my $self = shift;
-    my %args = (@_);
-    my $agreement = $args{'Agreement'} || $self->Agreement( @_ );
-    return undef unless $agreement and ref $agreement eq 'HASH';
+    my $meta = $RT::ServiceAgreements{'Levels'}{ $args{'Level'} };
+    return undef unless $meta;
 
-    my $res = $args{'Time'};
+    return $args{'Time'} if $meta->{'StartImmediately'};
 
-    my $ok = eval {
-        local $ENV{'TZ'} = $ENV{'TZ'};
-        if ( $agreement->{'Timezone'} && $agreement->{'Timezone'} ne ($ENV{'TZ'}||'') ) {
-            $ENV{'TZ'} = $agreement->{'Timezone'};
-            require POSIX; POSIX::tzset();
-        }
-
-        my $bhours = $self->BusinessHours( $agreement->{'BusinessHours'} );
-
-        if ( $agreement->{'OutOfHours'} && $bhours->first_after( $res ) != $res ) {
-            foreach ( qw(RealMinutes BusinessMinutes) ) {
-                next unless my $mod = $agreement->{'OutOfHours'}{ $_ };
-                ($agreement->{ $_ } ||= 0) += $mod;
-            }
-        }
-
-        if ( defined $agreement->{'BusinessMinutes'} ) {
-            if ( $agreement->{'BusinessMinutes'} ) {
-                $res = $bhours->add_seconds(
-                    $res, 60 * $agreement->{'BusinessMinutes'},
-                );
-            }
-            else {
-                $res = $bhours->first_after( $res );
-            }
-        }
-        $res += 60 * $agreement->{'RealMinutes'}
-            if defined $agreement->{'RealMinutes'};
-        1;
-    };
-
-    POSIX::tzset() if $agreement->{'Timezone'}
-        && $agreement->{'Timezone'} ne ($ENV{'TZ'}||'');
-    die $@ unless $ok;
-
-    return $res;
+    my $bhours = $self->BusinessHours( $meta->{'BusinessHours'} );
+    return $bhours->first_after( $args{'Time'} );
 }
 
 sub GetCustomField {
@@ -478,10 +384,8 @@ sub GetDefaultServiceLevel {
         else {
             return $args{'Queue'}->SLA if $args{'Queue'}->SLA;
         }
-
-        if ( my $info = $RT::ServiceAgreements{'QueueDefault'}{ $args{'Queue'}->Name } ) {
-            return $info unless ref $info;
-            return $info->{'Level'} || $RT::ServiceAgreements{'Default'};
+        if ( $RT::ServiceAgreements{'QueueDefault'} && $RT::ServiceAgreements{'QueueDefault'}{ $args{'Queue'}->Name } ) {
+            return $RT::ServiceAgreements{'QueueDefault'}{ $args{'Queue'}->Name };
         }
     }
     return $RT::ServiceAgreements{'Default'};
@@ -489,37 +393,35 @@ sub GetDefaultServiceLevel {
 
 =head1 TODO
 
-    * [implemented, TODO: tests for options in the config] default SLA for queues
+    * default SLA for queues
+    ** implemented
+    ** TODO: tests for options in the config
 
-    * [implemented, TODO: tests] add support for multiple b-hours definitions,
-      this could be very helpfull when you have 24/7 mixed with 8/5 and/or
-      something like 8/5+4/2 for different tickets(by requestor, queue or
-      something else). So people would be able to handle tickets in the right
-      order using Due dates.
+    * add support for multiple b-hours definitions, this could be very helpfull
+      when you have 24/7 mixed with 8/5 and/or something like 8/5+4/2 for different
+      tickets(by requestor, queue or something else). So people would be able to
+      handle tickets in the right order using Due dates.
+    ** implemented
+    ** TODO: tests
 
-    * [not implemented] WebUI
+    * WebUI
+    ** not implemented
 
 =head1 DESIGN
 
 =head2 Classes
 
 Actions are subclasses of L<RT::Action::SLA> class that is subclass of
-L<RT::Extension::SLA> and L<RT::Action> classes.
+L<RT::Extension::SLA> and L<RT::Action::Generic> classes.
 
 Conditions are subclasses of L<RT::Condition::SLA> class that is subclass of
-L<RT::Extension::SLA> and L<RT::Condition> classes.
+L<RT::Extension::SLA> and L<RT::Condition::Generic> classes.
 
 L<RT::Extension::SLA> is a base class for all classes in the extension,
 it provides access to config, generates L<Business::Hours> objects, and
 other things useful for whole extension. As this class is the base for
 all actions and conditions then we MUST avoid adding methods which overload
-methods in 'RT::{Condition,Action}' RT's modules.
-
-=head1 NOTES
-
-If you run C<make initdb> more than once you will create multiple SLA CFs.  You
-can remove these via RT's C<Configuration-E<gt>Global> menu, (both Custom Fields
-and Scrips).
+methods in 'RT::{Condition,Action}::Generic' RT's modules.
 
 =head1 AUTHOR
 
@@ -527,7 +429,7 @@ Ruslan Zakirov E<lt>ruz@bestpractical.comE<gt>
 
 =head1 COPYRIGHT
 
-This extension is Copyright (C) 2007-2009 Best Practical Solutions, LLC.
+This extension is Copyright (C) 2007-2008 Best Practical Solutions, LLC.
 
 It is freely redistributable under the terms of version 2 of the GNU GPL.
 
